@@ -1,9 +1,9 @@
 package Net::DNS::RR;
 
 #
-# $Id: RR.pm 1694 2018-07-16 04:19:40Z willem $
+# $Id: RR.pm 1726 2018-12-15 12:59:56Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1694 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1726 $)[1];
 
 
 =head1 NAME
@@ -14,12 +14,12 @@ Net::DNS::RR - DNS resource record base class
 
     use Net::DNS;
 
-    $rr = new Net::DNS::RR('example.com IN A 192.0.2.99');
+    $rr = new Net::DNS::RR('example.com IN AAAA 2001:DB8::1');
 
     $rr = new Net::DNS::RR(
 	    owner   => 'example.com',
-	    type    => 'A',
-	    address => '192.0.2.99'
+	    type    => 'AAAA',
+	    address => '2001:DB8::1'
 	    );
 
 
@@ -39,7 +39,6 @@ use Carp;
 use constant LIB => grep $_ ne '.', grep !ref($_), @INC;
 
 use Net::DNS::Parameters;
-use Net::DNS::Domain;
 use Net::DNS::DomainName;
 
 
@@ -67,7 +66,7 @@ sub new {
 
 =head2 new (from string)
 
-    $a	   = new Net::DNS::RR('host.example.com. 86400 A 192.0.2.1');
+    $aaaa  = new Net::DNS::RR('host.example.com. 86400 AAAA 2001:DB8::1');
     $mx	   = new Net::DNS::RR('example.com. 7200 MX 10 mailhost.example.com.');
     $cname = new Net::DNS::RR('www.example.com 300 IN CNAME host.example.com');
     $txt   = new Net::DNS::RR('txt.example.com 3600 HS TXT "text data"');
@@ -107,17 +106,17 @@ sub _new_string {
 
 	croak 'unable to parse RR string' unless scalar @token;
 	my $t1 = uc $token[0];
-	my $t2 = uc $token[1] if $#token;
+	my $t2 = $token[1];
 
 	my ( $ttl, $class );
 	if ( not defined $t2 ) {				# <owner> <type>
 		@token = ('ANY') if $classbyname{$t1};		# <owner> <class>
+	} elsif ( $t1 =~ /^\d/ ) {
+		$ttl = shift @token;				# <owner> <ttl> [<class>] <type>
+		$class = shift @token if $classbyname{uc $t2} || $t2 =~ /^CLASS\d/i;
 	} elsif ( $classbyname{$t1} || $t1 =~ /^CLASS\d/ ) {
 		$class = shift @token;				# <owner> <class> [<ttl>] <type>
 		$ttl = shift @token if $t2 =~ /^\d/;
-	} elsif ( $t1 =~ /^\d/ ) {
-		$ttl = shift @token;				# <owner> <ttl> [<class>] <type>
-		$class = shift @token if $classbyname{$t2} || $t2 =~ /^CLASS\d/;
 	}
 
 	my $type      = shift(@token);
@@ -152,8 +151,8 @@ sub _new_string {
 	    owner   => 'host.example.com',
 	    ttl	    => 86400,
 	    class   => 'IN',
-	    type    => 'A',
-	    address => '192.0.2.1'
+	    type    => 'AAAA',
+	    address => '2001:DB8::1'
 	    );
  
     $rr = new Net::DNS::RR(
@@ -269,7 +268,7 @@ sub encode {
 
 	my $owner = $self->{owner}->encode( $offset, @opaque );
 	my ( $type, $class, $ttl ) = @{$self}{qw(type class ttl)};
-	my $rdata = $self->_empty ? '' : $self->_encode_rdata( RRFIXEDSZ + length $owner, @opaque );
+	my $rdata = $self->_empty ? '' : $self->_encode_rdata( $offset + length($owner) + RRFIXEDSZ, @opaque );
 	pack 'a* n2 N n a*', $owner, $type, $class || 1, $ttl || 0, length $rdata, $rdata;
 }
 
@@ -292,7 +291,7 @@ sub canonical {
 
 	my $owner = $self->{owner}->canonical;
 	my ( $type, $class, $ttl ) = @{$self}{qw(type class ttl)};
-	my $rdata = $self->_empty ? '' : $self->_encode_rdata( RRFIXEDSZ + length $owner );
+	my $rdata = $self->_empty ? '' : $self->_encode_rdata( length($owner) + RRFIXEDSZ );
 	pack 'a* n2 N n a*', $owner, $type, $class || 1, $ttl || 0, length $rdata, $rdata;
 }
 
@@ -301,8 +300,8 @@ sub canonical {
 
     $rr->print;
 
-Prints the record to the standard output.  Calls the string method
-to get the formatted RR representation.
+Prints the resource record to the currently selected output filehandle.
+Calls the string method to get the formatted RR representation.
 
 =cut
 
@@ -315,10 +314,10 @@ sub print {
 
     print $rr->string, "\n";
 
-Returns a string representation of the RR using the zone file format
-described in RFC1035.  All domain names are fully qualified with
-trailing dot.  This differs from RR attribute methods, which omit
-the trailing dot.
+Returns a string representation of the RR using the master file format
+mandated by RFC1035.
+All domain names are fully qualified with trailing dot.
+This differs from RR attribute methods, which omit the trailing dot.
 
 =cut
 
@@ -349,9 +348,9 @@ sub string {
 
     $plain = $rr->plain;
 
-Returns a simplified single line representation of the RR using the
-zone file format defined in RFC1035.  This facilitates interaction
-with programs like nsupdate which have rudimentary RR parsers.
+Returns a simplified single-line representation of the RR.
+This facilitates interaction with programs like nsupdate
+which have rudimentary parsers.
 
 =cut
 
@@ -752,7 +751,7 @@ sub AUTOLOAD {				## Default method
 	no strict q/refs/;
 	my ($method) = reverse split /::/, $AUTOLOAD;
 	*{$AUTOLOAD} = sub {undef};	## suppress repetition and deep recursion
-	croak "$self has no class method '$method'" unless $oref;
+	croak qq[$self has no class method "$method"] unless $oref;
 
 	my $string = $self->string;
 	my @object = grep defined($_), $oref, $oref->VERSION;
@@ -760,7 +759,7 @@ sub AUTOLOAD {				## Default method
 	eval("require $module") if $oref eq __PACKAGE__;
 
 	@_ = ( <<"END", $@, "@object" );
-***  FATAL PROGRAM ERROR!!	Unknown instance method '$method'
+***  FATAL PROGRAM ERROR!!	Unknown instance method "$method"
 ***  which the program has attempted to call for the object:
 ***
 $string
