@@ -1,9 +1,9 @@
 package Net::DNS::Resolver::Base;
 
 #
-# $Id: Base.pm 1709 2018-09-07 08:03:09Z willem $
+# $Id: Base.pm 1727 2018-12-31 12:04:48Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1709 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1727 $)[1];
 
 
 #
@@ -26,7 +26,7 @@ our $VERSION = (qw$LastChangedRevision: 1709 $)[1];
 # [Revised March 2016, June 2018]
 
 
-use constant USE_SOCKET_IP => defined eval 'use IO::Socket::IP 0.32; 1;';
+use constant USE_SOCKET_IP => defined eval 'use IO::Socket::IP 0.32; 1';
 
 use constant IPv6 => USE_SOCKET_IP;
 
@@ -165,9 +165,9 @@ sub _untaint {
 sub _read_env {				## read resolver config environment variables
 	my $self = shift;
 
-	$self->nameservers( map split, $ENV{RES_NAMESERVERS} ) if defined $ENV{RES_NAMESERVERS};
+	$self->searchlist( map split, $ENV{LOCALDOMAIN} ) if defined $ENV{LOCALDOMAIN};
 
-	$self->domain( $ENV{LOCALDOMAIN} ) if defined $ENV{LOCALDOMAIN};
+	$self->nameservers( map split, $ENV{RES_NAMESERVERS} ) if defined $ENV{RES_NAMESERVERS};
 
 	$self->searchlist( map split, $ENV{RES_SEARCHLIST} ) if defined $ENV{RES_SEARCHLIST};
 
@@ -181,14 +181,14 @@ sub _read_config_file {			## read resolver config file
 	my $self = shift;
 	my $file = shift;
 
-	local *FILE;
-	open( FILE, $file ) or croak "$file: $!";
+	my $filehandle;
+	open( $filehandle, '<', $file ) or croak "$file: $!";
 
 	my @nameserver;
 	my @searchlist;
 
 	local $_;
-	while (<FILE>) {
+	while (<$filehandle>) {
 		s/[;#].*$//;					# strip comments
 
 		/^nameserver/ && do {
@@ -217,7 +217,7 @@ sub _read_config_file {			## read resolver config file
 		};
 	}
 
-	close(FILE);
+	close($filehandle);
 
 	$self->nameservers(@nameserver) if @nameserver;
 	$self->searchlist(@searchlist)	if @searchlist;
@@ -363,11 +363,7 @@ sub query {
 	my $self = shift;
 	my $name = shift || '.';
 
-	my @sfix;
-
-	if ( $self->{defnames} && ( ( $name =~ tr/././ ) < $self->{ndots} ) ) {
-		@sfix = $self->domain unless $name =~ m/:|\.\d*$/;
-	}
+	my @sfix = $self->{defnames} && ( $name !~ m/[.:]/ ) ? $self->domain : ();
 
 	my $fqdn = join '.', $name, @sfix;
 	$self->_diag( 'query(', $fqdn, @_, ')' );
@@ -382,11 +378,12 @@ sub search {
 	return $self->query(@_) unless $self->{dnsrch};
 
 	my $name = shift || '.';
+	my $dots = $name =~ tr/././;
 
-	my @sfix = ( $name =~ m/:|\.\d*$/ ) ? () : @{$self->{searchlist}};
-	my ( $domain, @etc ) = ( $name =~ tr/././ ) < $self->{ndots} ? (@sfix) : ( undef, @sfix );
+	my @sfix = ( $dots < $self->{ndots} ) ? @{$self->{searchlist}} : ();
+	my ( $one, @more ) = ( $name =~ m/:|\.\d*$/ ) ? () : ( $dots ? ( undef, @sfix ) : @sfix );
 
-	foreach my $suffix ( $domain, @etc ) {
+	foreach my $suffix ( $one, @more ) {
 		my $fqname = $suffix ? join( '.', $name, $suffix ) : $name;
 		$self->_diag( 'search(', $fqname, @_, ')' );
 		my $packet = $self->send( $fqname, @_ ) || next;
@@ -402,6 +399,8 @@ sub send {
 	my $packet	= $self->_make_query_packet(@_);
 	my $packet_data = $packet->data;
 
+	$self->_reset_errorstring;
+
 	return $self->_send_tcp( $packet, $packet_data )
 			if $self->{usevc} || length $packet_data > $self->_packetsz;
 
@@ -418,8 +417,6 @@ sub send {
 sub _send_tcp {
 	my ( $self, $query, $query_data ) = @_;
 
-	$self->_reset_errorstring;
-
 	my $tcp_packet = pack 'n a*', length($query_data), $query_data;
 	my @ns = $self->nameservers();
 	my $fallback;
@@ -465,8 +462,6 @@ sub _send_tcp {
 sub _send_udp {
 	my ( $self, $query, $query_data ) = @_;
 
-	$self->_reset_errorstring;
-
 	my @ns	    = $self->nameservers;
 	my $port    = $self->{port};
 	my $retrans = $self->{retrans} || 1;
@@ -485,8 +480,8 @@ NAMESERVER: foreach my $ns (@ns) {
 
 			# state vector replaces corresponding element of @ns array
 			unless ( ref $ns ) {
-				my $socket = $self->_create_udp_socket($ns) || next;
 				my $dst_sockaddr = $self->_create_dst_sockaddr( $ns, $port );
+				my $socket = $self->_create_udp_socket($ns) || next;
 				$ns = [$socket, $ns, $dst_sockaddr];
 			}
 
@@ -548,6 +543,8 @@ sub bgsend {
 	my $packet	= $self->_make_query_packet(@_);
 	my $packet_data = $packet->data;
 
+	$self->_reset_errorstring;
+
 	return $self->_bgsend_tcp( $packet, $packet_data )
 			if $self->{usevc} || length $packet_data > $self->_packetsz;
 
@@ -558,8 +555,6 @@ sub bgsend {
 sub _bgsend_tcp {
 	my ( $self, $packet, $packet_data ) = @_;
 
-	$self->_reset_errorstring;
-
 	my $tcp_packet = pack 'n a*', length($packet_data), $packet_data;
 
 	foreach my $ip ( $self->nameservers ) {
@@ -583,13 +578,11 @@ sub _bgsend_tcp {
 sub _bgsend_udp {
 	my ( $self, $packet, $packet_data ) = @_;
 
-	$self->_reset_errorstring;
-
 	my $port = $self->{port};
 
 	foreach my $ip ( $self->nameservers ) {
-		my $socket = $self->_create_udp_socket($ip) || next;
 		my $dst_sockaddr = $self->_create_dst_sockaddr( $ip, $port );
+		my $socket = $self->_create_udp_socket($ip) || next;
 
 		$self->_diag( 'bgsend', "[$ip]:$port" );
 
@@ -685,9 +678,8 @@ sub _accept_reply {
 	my $header = $reply->header;
 	return unless $header->qr;
 
-	return 1 unless $query;					# SpamAssassin 3.4.1 workaround
+	return if $query && $header->id != $query->header->id;
 
-	return unless $header->id == $query->header->id;
 	$self->errorstring( $header->rcode );			# historical quirk
 }
 
@@ -884,7 +876,6 @@ sub _create_tcp_socket {
 				unless $ip6_addr;
 	}
 
-	$self->errorstring("no socket $sock_key $!") unless $socket;
 	$self->{persistent}{$sock_key} = $self->{persistent_tcp} ? $socket : undef;
 	return $socket;
 }
@@ -917,7 +908,6 @@ sub _create_udp_socket {
 				unless $ip6_addr;
 	}
 
-	$self->errorstring("no socket $sock_key $!") unless $socket;
 	$self->{persistent}{$sock_key} = $self->{persistent_udp} ? $socket : undef;
 	return $socket;
 }
@@ -1123,7 +1113,7 @@ sub AUTOLOAD {				## Default method
 
 	my $name = $AUTOLOAD;
 	$name =~ s/.*://;
-	croak "$name: no such method" unless $public_attr{$name};
+	croak qq[unknown method "$name"] unless $public_attr{$name};
 
 	no strict q/refs/;
 	*{$AUTOLOAD} = sub {
